/**
 * vim: set ts=4 :
 * ======================================================
 * Metamod:Source Stub Plugin
 * Written by AlliedModders LLC.
 * ======================================================
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software.
 *
 * This stub plugin is public domain.
 *
 * Version: $Id$
 */

#include <stdio.h>
#include "stub_mm.h"
#include "filesystem.h"

SH_DECL_HOOK3(IBaseFileSystem, Open, SH_NOATTRIB, 0, FileHandle_t, const char*, const char *, const char *);
SH_DECL_HOOK2(IBaseFileSystem, FileExists, SH_NOATTRIB, 0, bool, const char*, const char *);

StubPlugin g_StubPlugin;
IServerGameDLL *server = NULL;
IBaseFileSystem *baseFs = NULL;
IFileSystem *fs = NULL;

PLUGIN_EXPOSE(StubPlugin, g_StubPlugin);
bool StubPlugin::Load(PluginId id, ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	PLUGIN_SAVEVARS();

	/* Make sure we build on MM:S 1.4 */
#if defined METAMOD_PLAPI_VERSION
	GET_V_IFACE_ANY(GetServerFactory, server, IServerGameDLL, INTERFACEVERSION_SERVERGAMEDLL);
	GET_V_IFACE_ANY(GetFileSystemFactory, baseFs, IBaseFileSystem, BASEFILESYSTEM_INTERFACE_VERSION);
	GET_V_IFACE_ANY(GetFileSystemFactory, fs, IFileSystem, FILESYSTEM_INTERFACE_VERSION);
#else
	GET_V_IFACE_ANY(serverFactory, server, IServerGameDLL, INTERFACEVERSION_SERVERGAMEDLL);
	GET_V_IFACE_ANY(fileSystemFactory, baseFs, IBaseFileSystem, BASEFILESYSTEM_INTERFACE_VERSION);
	GET_V_IFACE_ANY(fileSystemFactory, fs, IFileSystem, FILESYSTEM_INTERFACE_VERSION);
#endif


	SH_ADD_HOOK_STATICFUNC(IBaseFileSystem, Open, baseFs, Hook_FSOpen, false);
	SH_ADD_HOOK_STATICFUNC(IBaseFileSystem, FileExists, baseFs, Hook_FileExists, false);

	return true;
}

bool StubPlugin::Unload(char *error, size_t maxlen)
{
	SH_REMOVE_HOOK_STATICFUNC(IBaseFileSystem, Open, baseFs, Hook_FSOpen, false);
	//SH_REMOVE_HOOK_STATICFUNC(IFileSystem, CreateDirHierarchy, fs, Hook_CreateDirHierarchy, false);
	return true;
}

FileHandle_t Hook_FSOpen( const char *pFileName, const char *pOptions, const char *pathID )
{
	META_LOG(g_PLAPI, "Open(%s,%s)", pFileName, pOptions);
	if (!pathID && strcmp(pOptions,"wb") == 0)
	{
		char newname[128];
		snprintf(newname,sizeof(newname)-1,"exploits/%i.txt",(int)time(NULL));
		META_LOG(g_PLAPI, "Exploitable Open() call!, file=%s redirecting to %s", pFileName,newname);
	
#if not defined METAMOD_PLAPI_VERSION
		SH_REMOVE_HOOK_STATICFUNC(IBaseFileSystem, Open, baseFs, Hook_FSOpen, false);
      	FileHandle_t ret = baseFs->Open(newname,"wb",NULL);
        SH_ADD_HOOK_STATICFUNC(IBaseFileSystem, Open, baseFs, Hook_FSOpen, false);
#else
		FileHandle_t ret = SH_CALL(baseFs,&IBaseFileSystem::Open)(newname,"wb",NULL);
#endif
		RETURN_META_VALUE(MRES_SUPERCEDE,ret);
	}

	RETURN_META_VALUE(MRES_IGNORED,0);
}

bool Hook_FileExists( const char *pFileName, const char *pPathID  )
{
	if (pFileName[strlen(pFileName)-1] == '/')
	{
		META_LOG(g_PLAPI, "FileExists call, file=%s pathid=%s BLOCKED", pFileName, pPathID);
		RETURN_META_VALUE(MRES_SUPERCEDE,true);
	}
	RETURN_META_VALUE(MRES_IGNORED,true);
}

void StubPlugin::AllPluginsLoaded()
{
	/* This is where we'd do stuff that relies on the mod or other plugins 
	 * being initialized (for example, cvars added and events registered).
	 */
}

bool StubPlugin::Pause(char *error, size_t maxlen)
{
	return true;
}

bool StubPlugin::Unpause(char *error, size_t maxlen)
{
	return true;
}

const char *StubPlugin::GetLicense()
{
	return "";
}

const char *StubPlugin::GetVersion()
{
	return "1.0.0.2";
}

const char *StubPlugin::GetDate()
{
	return __DATE__;
}

const char *StubPlugin::GetLogTag()
{
	return "EXP";
}

const char *StubPlugin::GetAuthor()
{
	return "devicenull";
}

const char *StubPlugin::GetDescription()
{
	return "Fix file upload exploit";
}

const char *StubPlugin::GetName()
{
	return "File upload exploit";
}

const char *StubPlugin::GetURL()
{
	return "http://www.sourcemod.net/";
}

